﻿#region usings
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using IdeaBlade.EntityModel;

using DomainModel;
#endregion

namespace FirstSilverlightApp {

  public partial class Page : UserControl {

    #region ctor & load

    public Page() {
      InitializeComponent();
      Loaded += Page_Loaded;
    }

    void Page_Loaded(object sender, RoutedEventArgs e) {
      CreateEntityManager(); // the gateway to persistence
      InitializeQueries();
      ConfigureDataGrid();
      Reset();
    }

    private void ConfigureDataGrid() {
      dg.ItemsSource = _entities;
    }

    private void CreateEntityManager() {
      WriteMessage("Creating EntityManager ...");
      _entityManager = new DomainModelEntityManager(false);
    }


    #endregion

    #region Button Click Handlers

    private void btnConnect_Click(object sender, RoutedEventArgs e) {
      Connect(); // to the server
    }

    private void btnLogin_Click(object sender, RoutedEventArgs e) {
      Login(); // for security and identification
    }

    private void btnFetch_Click(object sender, RoutedEventArgs e) {
      Fetch(); // Get from the database
    }

    private void btnView_Click(object sender, RoutedEventArgs e) {
      View(false); // View the selected entity (user may select to edit)
    }

    private void btnEdit_Click(object sender, RoutedEventArgs e) {
      Edit(); // Edit the selected entity
    }

    private void btnAdd_Click(object sender, RoutedEventArgs e) {
      Add(); // Add a new entity locally
    }

    private void btnDelete_Click(object sender, RoutedEventArgs e) {
      Delete(); // Delete a new entity locally
    }

    private void btnSave_Click(object sender, RoutedEventArgs e) {
      Save();
    }

    private void btnLogout_Click(object sender, RoutedEventArgs e) {
      Logout();
    }

    private void btnReset_Click(object sender, RoutedEventArgs e) {
      Reset();
    }
    #endregion

    #region Initialize Queries

    private void InitializeQueries() {
      AddQueries();
      InitializeQueryCombo();
    }

    private void InitializeQueryCombo() {
      var qNames = _queries.Keys.ToList();
      queryCombo.ItemsSource = qNames;
      queryCombo.SelectedItem = qNames.FirstOrDefault();
      queryCombo.SelectionChanged += delegate { Fetch(); };
    }

    #endregion

    #region Connect

    private void Connect() {

      _entityManager.ConnectAsync(
        args => {
          if (args.HasError) {
            WriteMessage(args.Error.Message);
            args.MarkErrorAsHandled();
          }
          else {
            WriteMessage("Connected");
          }
          btnLogin.IsEnabled = args.IsCompleted;
          if (args.CompletedSynchronously) return;
        },
        null);

      if (!_entityManager.IsConnected) WriteMessage("Connecting ...");
    }

    #endregion

    #region Login

    private void Login() {
      var cred = new LoginCredential("demo", "demo", "demo");

      WriteMessage("Logging in ...");

      _entityManager.LoginAsync(
        cred,
        args => {
          if (args.HasError) {
            WriteMessage(args.Error.Message);
            args.MarkErrorAsHandled();
          }
          else {
            WriteMessage("Logged in");
          }
          queryCombo.IsEnabled = args.IsCompleted;
          btnFetch.IsEnabled = args.IsCompleted;
          btnSave.IsEnabled = args.IsCompleted;
          btnLogout.IsEnabled = args.IsCompleted;
        },
        null);
    }

    #endregion

    #region Logout

    private void Logout() {
      WriteMessage("Logging out ...");

      _entityManager.LogoutAsync(
        args => {
          WriteMessage("Logged out");
          queryCombo.IsEnabled = false;
          btnFetch.IsEnabled = false;
          btnSave.IsEnabled = false;
        },
        null);
    }

    #endregion

    #region Queries

    private void AddQueries() {

      _queries = new Dictionary<string, IEntityQuery>();

      _queries.Add("Get all Customers", _entityManager.Customers);

      _queries.Add("Get Customers starting with A",
                  _entityManager.Customers
                    .Where(c => c.CompanyName.StartsWith("A")));

      _queries.Add("Get Customers with Orders in Oregon",
                  _entityManager.Customers
                    .Where(c => c.Orders.Any(o => o.ShipRegion == "OR")));

      _queries.Add("Get Customers starting with B from cache-only",
                  _entityManager.Customers
                    .Where(c => c.CompanyName.StartsWith("B"))
                    .With(QueryStrategy.CacheOnly));
    }

    #endregion

    #region Fetch

    private void Fetch() {

      var query = GetQuery();
      if (null == query) {
        WriteMessage("No query selected; pick a query.");
        return;
      }

      WriteMessage("Fetching ...");

      _entityManager.ExecuteQueryAsync(
        query,
        args => {
          if (args.HasError) {
            WriteMessage(args.Error.Message);
            args.MarkErrorAsHandled();
          }
          else {
            RepopulateEntitiesCollection(args.Results);
            dg.ItemsSource = _entities;
            dg.SelectedIndex = 0;
            ReportFetchCount(args.Results);
          }
        },
        null);
    }

    private void RepopulateEntitiesCollection(IEnumerable results) {
      _entities.Clear();
      foreach (Entity anEntity in results) {
        _entities.Add(anEntity);
      }
    }

    private IEntityQuery GetQuery() {
      var queryName = (string)queryCombo.SelectedItem;
      return queryName == null ? null : _queries[queryName];
    }

    private void ReportFetchCount(IEnumerable result) {
      WriteMessage(string.Format("Retrieved {0} customers", ((ICollection)result).Count));
    }

    #endregion

    #region Reset

    private void Reset() {

      // Start all over again ...

      btnLogin.IsEnabled = false;
      btnFetch.IsEnabled = false;
      btnLogout.IsEnabled = false;
      btnSave.IsEnabled = false;
      queryCombo.IsEnabled = false;

      dg.ItemsSource = null;

      _entityManager.Clear();
      if (_entityManager.IsLoggedIn) Logout();
      if (_entityManager.IsConnected) _entityManager.Disconnect();

      WriteMessage("Ready to connect ...");
    }

    #endregion

    #region View / Edit / Add / Delete

    // View the selected entity (allows segue into Edit mode)
    private void View(bool enterEditMode) {
      Entity selectedEntity = (Entity)dg.SelectedItem;
      EditCustomerChildWindow editWindow = new EditCustomerChildWindow();
      editWindow.df.DataContext = dg.DataContext;
      editWindow.df.CurrentItem = selectedEntity;
      if (enterEditMode) {
        editWindow.df.BeginEdit();
      }
      editWindow.Show();
    }

    // Edit the selected entity
    private void Edit() {
      // Add a new entity and select it
      View(true);
    }

    private void Add() {
      // Add a new entity and select it
      Customer newCustomer = Customer.Create(_entityManager, "[Company Name]");
      _entities.Add(newCustomer);
      dg.SelectedItem = newCustomer;
      Edit();
    }

    private void Delete() {
      // Delete selected entity and remove from list
      Customer selectedCustomer = (Customer)dg.SelectedItem;
      selectedCustomer.DeleteGraph();      // Careful!  This deletes customer, orders and order details.
                                           // Note that InternationalOrders won't be deleted, and will cause an error.
                                           // In general, use caution with what you delete and how you cascade the deletion in the object graph.
      _entities.Remove(selectedCustomer);
    }

    #endregion View / Edit / Add

    #region Save

    private void Save() {

      if (!_entityManager.HasChanges()) {
        WriteMessage("No changes to save.");
        return;
      }

      WriteMessage("Saving ...");

      _entityManager.SaveChangesAsync(
        args => {
          if (args.CompletedSuccessfully) {
            WriteMessage("Changes saved");
          }
          else {
            WriteMessage(args.Error.Message);
            args.MarkErrorAsHandled();
          }
        },
        null);
    }

    #endregion

    #region Misc

    private void WriteMessage(string msg) {
      txtStatus.Text = msg;
    }

    #endregion

    #region Private Fields

    private DomainModelEntityManager _entityManager;
    private Dictionary<string, IEntityQuery> _queries;
    ObservableCollection<Entity> _entities = new ObservableCollection<Entity>();

    #endregion Private Fields
  }
}
