﻿#Region "usings"
Imports System.Collections
Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.Linq
Imports System.Windows
Imports System.Windows.Controls
Imports IdeaBlade.EntityModel

#End Region


Partial Public Class Page
    Inherits UserControl

#Region "ctor & load"

    Public Sub New()
        InitializeComponent()
        AddHandler Loaded, AddressOf Page_Loaded
    End Sub

    Private Sub Page_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
        CreateEntityManager()
        ' the gateway to persistence
        InitializeQueries()
        ConfigureDataGrid()
        Reset()
    End Sub

    Private Sub ConfigureDataGrid()
        dg.ItemsSource = _entities
    End Sub

    Private Sub CreateEntityManager()
        WriteMessage("Creating EntityManager ...")
        _entityManager = New DomainModelEntityManager(False)
    End Sub


#End Region

#Region "Button Click Handlers"

    Private Sub btnConnect_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Connect()         ' to the server
    End Sub

    Private Sub btnLogin_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Login()         ' for security and identification
    End Sub

    Private Sub btnFetch_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Fetch()         ' Get from the database
    End Sub

    Private Sub btnView_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        View(False)         ' View the selected entity (user may select to edit)
    End Sub

    Private Sub btnEdit_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Edit()         ' Edit the selected entity
    End Sub

    Private Sub btnAdd_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Add()         ' Add a new entity locally
    End Sub

    Private Sub btnDelete_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Delete()         ' Delete a new entity locally
    End Sub

    Private Sub btnSave_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Save()
    End Sub

    Private Sub btnLogout_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Logout()
    End Sub

    Private Sub btnReset_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Reset()
    End Sub
#End Region

#Region "Initialize Queries"

    Private Sub InitializeQueries()
        AddQueries()
        InitializeQueryCombo()
    End Sub

    Private Sub InitializeQueryCombo()
        Dim qNames = _queries.Keys.ToList()
        queryCombo.ItemsSource = qNames
        queryCombo.SelectedItem = qNames.FirstOrDefault()
        AddHandler queryCombo.SelectionChanged, Sub()
                                                    Fetch()
                                                End Sub
    End Sub

#End Region

#Region "Connect"

    Private Sub Connect()

        _entityManager.ConnectAsync(Sub(args)
                                        If args.Error IsNot Nothing Then
                                            WriteMessage(args.Error.Message)
                                        Else
                                            WriteMessage("Connected")
                                        End If
                                        btnLogin.IsEnabled = args.IsCompleted
                                        If args.CompletedSynchronously Then
                                            Return
                                        End If
                                    End Sub, Nothing)

        If Not _entityManager.IsConnected Then
            WriteMessage("Connecting ...")
        End If
    End Sub

#End Region

#Region "Login"

    Private Sub Login()
        Dim cred = New LoginCredential("demo", "demo", "demo")

        WriteMessage("Logging in ...")

        _entityManager.LoginAsync(cred, Sub(args)
                                            If args.Error IsNot Nothing Then
                                                WriteMessage(args.Error.Message)
                                            Else
                                                WriteMessage("Logged in")
                                            End If
                                            queryCombo.IsEnabled = args.IsCompleted
                                            btnFetch.IsEnabled = args.IsCompleted
                                            btnSave.IsEnabled = args.IsCompleted
                                            btnLogout.IsEnabled = args.IsCompleted
                                        End Sub, Nothing)
    End Sub

#End Region

#Region "Logout"

    Private Sub Logout()
        WriteMessage("Logging out ...")

        _entityManager.LogoutAsync(Sub(args)
                                       WriteMessage("Logged out")
                                       queryCombo.IsEnabled = False
                                       btnFetch.IsEnabled = False
                                       btnSave.IsEnabled = False
                                   End Sub, Nothing)
    End Sub

#End Region

#Region "Queries"

    Private Sub AddQueries()

        _queries = New Dictionary(Of String, IEntityQuery)()

        _queries.Add("Get all Customers", _entityManager.Customers)

        _queries.Add("Get Customers starting with A", _entityManager.Customers.Where(Function(c) c.CompanyName.StartsWith("A")))

        _queries.Add("Get Customers with Orders in Oregon", _entityManager.Customers.Where(Function(c) c.Orders.Any(Function(o) o.ShipRegion = "OR")))

        _queries.Add("Get Customers starting with B from cache-only", _entityManager.Customers.Where(Function(c) c.CompanyName.StartsWith("B")).[With](QueryStrategy.CacheOnly))
    End Sub

#End Region

#Region "Fetch"

    Private Sub Fetch()

        Dim query = GetQuery()
        If query Is Nothing Then
            WriteMessage("No query selected; pick a query.")
            Return
        End If

        WriteMessage("Fetching ...")

        _entityManager.ExecuteQueryAsync(query, Sub(args)
                                                    If args.Error IsNot Nothing Then
                                                        WriteMessage(args.Error.Message)
                                                    Else
                                                        RepopulateEntitiesCollection(args.Results)
                                                        dg.ItemsSource = _entities
                                                        dg.SelectedIndex = 0
                                                        ReportFetchCount(args.Results)
                                                    End If
                                                End Sub, Nothing)
    End Sub

    Private Sub RepopulateEntitiesCollection(ByVal results As IEnumerable)
        _entities.Clear()
        For Each anEntity As Entity In results
            _entities.Add(anEntity)
        Next
    End Sub

    Private Function GetQuery() As IEntityQuery
        Dim queryName = DirectCast(queryCombo.SelectedItem, String)
        Return If(queryName Is Nothing, Nothing, _queries(queryName))
    End Function

    Private Sub ReportFetchCount(ByVal result As IEnumerable)
        WriteMessage(String.Format("Retrieved {0} customers", DirectCast(result, ICollection).Count))
    End Sub

#End Region

#Region "Reset"

    Private Sub Reset()

        ' Start all over again ...

        btnLogin.IsEnabled = False
        btnFetch.IsEnabled = False
        btnLogout.IsEnabled = False
        btnSave.IsEnabled = False
        queryCombo.IsEnabled = False

        dg.ItemsSource = Nothing

        _entityManager.Clear()
        If _entityManager.IsLoggedIn Then
            Logout()
        End If
        If _entityManager.IsConnected Then
            _entityManager.Disconnect()
        End If

        WriteMessage("Ready to connect ...")
    End Sub

#End Region

#Region "View / Edit / Add / Delete"

    ' View the selected entity (allows segue into Edit mode)
    Private Sub View(ByVal enterEditMode As Boolean)
        Dim selectedEntity As Entity = DirectCast(dg.SelectedItem, Entity)
        Dim editWindow As New EditCustomerChildWindow()
        AddHandler editWindow.Closed, New System.EventHandler(AddressOf editWindow_Closed)
        editWindow.df.DataContext = dg.DataContext
        editWindow.df.CurrentItem = selectedEntity
        If enterEditMode Then
            editWindow.df.BeginEdit()
        End If
        editWindow.Show()
    End Sub

    ' Edit the selected entity
    Private Sub Edit()
        ' Add a new entity and select it
        View(True)
    End Sub

    Private Sub Add()
        ' Add a new entity and select it
        Dim newCustomer As Customer = Customer.Create(_entityManager, "[Company Name]")
        _entities.Add(newCustomer)
        dg.SelectedItem = newCustomer
        Edit()
    End Sub

    Private Sub Delete()
        ' Delete selected entity and remove from list
        Dim selectedCustomer As Customer = DirectCast(dg.SelectedItem, Customer)
        'selectedCustomer.DeleteGraph();  // use this one to delete the whole object graph
        selectedCustomer.EntityAspect.Delete()
        _entities.Remove(selectedCustomer)
    End Sub

    Private Sub editWindow_Closed(ByVal sender As Object, ByVal e As System.EventArgs)

        'EditCustomerChildWindow editWindow = (EditCustomerChildWindow)sender;

        'if ((bool)editWindow.DialogResult) {
        '    editWindow.DialogResult.Text = "User clicked OK in child window.";
        '}
        'else {
        '    editWindow.DialogResult.Text = "User clicked Cancel in child window.";
        '}
    End Sub

#End Region

#Region "Save"

    Private Sub Save()

        If Not _entityManager.HasChanges() Then
            WriteMessage("No changes to save.")
            Return
        End If

        WriteMessage("Saving ...")

        _entityManager.SaveChangesAsync(Sub(args)
                                            If args.IsCompleted Then
                                                WriteMessage("Changes saved")
                                            Else
                                                WriteMessage(args.Error.Message)
                                            End If
                                        End Sub, Nothing)
    End Sub

#End Region

#Region "Misc"

    Private Sub WriteMessage(ByVal msg As String)
        txtStatus.Text = msg
    End Sub

#End Region

#Region "Private Fields"

    Private _entityManager As DomainModelEntityManager
    Private _queries As Dictionary(Of String, IEntityQuery)
    Private _entities As New ObservableCollection(Of Entity)()

#End Region
End Class